def action(ns):
    print ns.t.bold_magenta(u"Compacting local databases")
    print u""

    for name, db in ns.databases.items():
        print ns.t.cyan(u"Compacting database: {0}".format(name))
        stats = db.compact()
        print ns.t.cyan(u"  pruned {0.noops} non-operation entries".format(
            stats))

    return 0


def setup(parser):
    parser.set_defaults(action=action)



import datetime

from .utils import format_datetime


class SeriesDAO(object):
    def __init__(self, db, series_db, episodes_db, watched):
        self._db = db
        self._series_db = series_db
        self._episodes_db = episodes_db
        self._watched = watched

    def add(self, series):
        self._db.list_append("series", series['id'])
        self._series_db[str(series['id'])] = series

    def remove(self, series):
        self._db.list_remove("series", series['id'])
        del self._series_db[str(series['id'])]

    def has_series(self, series):
        return series['id'] in self._db.get("series", [])

    def list_series(self):
        result = list()

        for series_id in self._db.get("series", []):
            series = self._series_db.get(str(series_id))

            if not series:
                continue

            result.append(series)

        return result

    def find_series(self, series_query):
        result = list()

        series_query = series_query.lower()

        for series_id in self._db.get("series", []):
            series = self._series_db.get(str(series_id))

            if not series:
                continue

            if series_query not in series['series_name'].lower():
                continue

            result.append(series)

        return result

    def set_episodes(self, series, episodes):
        self._episodes_db[str(series['id'])] = episodes

    def get(self, series_id):
        return self._series_db.get(str(series_id))

    def get_episodes(self, series):
        return self._episodes_db.get(str(series['id']))

    def get_season_episodes(self, series, season_number):
        results = list()

        for episode in self._episodes_db.get(str(series['id'])):
            if episode['season_number'] != season_number:
                continue

            results.append(episode)

        return results

    def is_episode_watched(self, episode):
        return str(episode['id']) in self._watched

    def set_episode_watched(self, episode, watched=True):
        now = datetime.datetime.now()

        if watched:
            self._watched[str(episode['id'])] = format_datetime(now)
        else:
            del self._watched[str(episode['id'])]



import datetime

from ..utils import find_next_episode
from ..utils import numeric_ranges
from ..utils import numeric_range
from ..utils import with_resource
from ..utils import local_series_finder

from ..format import short_episode
from ..format import format_airdate
from ..format import format_days


def episode_key(now):
    def __episode_key(item):
        season, episode, airdate = item

        if episode is None or airdate is None:
            return 0

        return -((now - airdate).days)

    return __episode_key


def filter_episodes(ns, next_episodes, specific, now):
    before, after = ns.range

    for s, episode, airdate in next_episodes:
        delta_days = abs((now - airdate).days)

        if after is not None and delta_days > after:
            if specific or ns.all:
                color = ns.C.range_outside
            else:
                continue
        elif before is None or delta_days < before:
            color = ns.C.range_before
        else:
            color = ns.C.range_inside

        yield color, s, episode


@with_resource(local_series_finder)
def action(ns, series):
    specific = False

    if series is None:
        series = ns.series.list_series()
    else:
        specific = True
        series = [series]

    before, after = ns.range

    # prevent excessive time fetching.
    now = datetime.datetime.now()

    episode_sort_key = episode_key(now)

    next_episodes = list()
    all_seen = list()

    print ns.C.title(u"Next episodes to watch out for")

    if specific or ns.all:
        print (u"Airing within {0}, within {1}, "
               "outside {2} and {3}").format(
                   ns.C.range_before(format_days(before)),
                   ns.C.range_inside(format_days(after)),
                   ns.C.range_outside(format_days(after)),
                   ns.C.all_seen("never"))
    else:
        print u"Airing within {0} and within {1}".format(
            ns.C.range_before(format_days(before)),
            ns.C.range_inside(format_days(after)))

    print u""

    for s in series:
        episodes = ns.series.get_episodes(s)

        if episodes is None:
            print ns.C.warning(u"episodes not synced: {0}".format(
                s['series_name']))
            continue

        result = find_next_episode(
            episodes, ns.series.is_episode_watched,
            ignored_seasons=ns.ignore)

        if result is None:
            all_seen.append(s)
            continue

        next_episode, next_airdate = result
        next_episodes.append((s, next_episode, next_airdate))

    next_episodes = sorted(next_episodes, key=episode_sort_key)
    next_episodes = filter_episodes(ns, next_episodes, specific, now)

    group = None

    for color, series, episode in next_episodes:
        aired = episode['first_aired']

        if group != aired:
            print color(u"{0}".format(format_airdate(aired, now=now)))
            group = aired

        print u"  {0} {1}".format(
            ns.C.series_title(series['series_name']),
            short_episode(episode))

    if specific or ns.all:
        print color(ns.C.all_seen(u"never (all seen)"))

        for s in all_seen:
            print u"  {0}".format(
                ns.C.series_title(s['series_name']))

    return 0


def setup(parser):
    parser.add_argument(
        "series_query",
        metavar="<name|id>",
        nargs='?',
        default=None,
        help="The id or name of the series.",
    )

    parser.add_argument(
        '--ignore',
        '-i',
        metavar="<ranges>",
        help="Specify a set of seasons to ignore, defaults to '0'",
        default=set([0]),
        type=numeric_ranges,
    )

    parser.add_argument(
        '--all',
        '-a',
        help=("Display all series episodes, regardless of when they are "
              "airing."),
        default=False,
        action='store_const',
        const=True,
    )

    parser.add_argument(
        '--range',
        help=("Specify how many days ago are relevant, these will show up in "
              "a different color, default: 30"),
        default=(30, 90),
        type=numeric_range,
    )

    parser.set_defaults(action=action)



from ..utils import api_series_finder
from ..utils import with_resource


@with_resource(api_series_finder)
def action(ns, series):
    if ns.series.has_series(series):
        print ns.t.bold_red(
            "Already exists: {0}".format(
                series['series_name']))
        return 0

    ns.series.add(series)

    print ns.t.bold_green(u"added: {0}".format(series['series_name']))
    return 0


def setup(parser):
    parser.add_argument(
        "series_query",
        metavar="<id|name>",
        help="The id of the series to add",
    )

    parser.set_defaults(action=action)



import yaml
import functools
import datetime

DATETIME_FORMAT = "%Y-%m-%d"


def write_yaml(fp, document):
    """
    Write a document to the specified file descriptor that is formatted in a
    readable way.
    """
    yaml.dump(document, fp, default_flow_style=False)


def read_yaml(fp):
    return yaml.load(fp)


def format_datetime(d):
    return d.strftime(DATETIME_FORMAT)


def parse_datetime(s):
    return datetime.datetime.strptime(s, DATETIME_FORMAT)


def pick_one(alternatives, title="Pick one: ", indent="  "):
    while True:
        for i, (alternative, _) in enumerate(alternatives):
            print u"{0}{1}) {2}".format(indent, i, alternative)

        try:
            user_input = raw_input(title)
        except EOFError:
            print "No input"
            return None

        try:
            user_input = int(user_input)
        except:
            print "Invalid input '{0}'".format(user_input)
            continue

        if user_input < 0:
            return None

        try:
            _, data = alternatives[user_input]
        except IndexError:
            print "Invalid index {0}, try one between 0-{1}".format(
                user_input, len(alternatives) - 1)
            continue

        return data


def find_series_external(fetch_series, series_id):
    results = fetch_series(series_id)

    if len(results) <= 0:
        return None

    if len(results) == 1:
        return results[0]['id']

    alternatives = [
        (u"{0[series_name]} ({0[id]})".format(s), s['id']) for s in results]

    return pick_one(
        alternatives,
        title="Pick a series (ctrl-d to abort): ")


def find_series(fetch_series, series_id):
    try:
        return int(series_id)
    except:
        return find_series_external(fetch_series, series_id)


def group_episodes(episodes):
    seasons = dict()

    for episode in episodes:
        season_number = episode.get("season_number", 0)
        seasons.setdefault(season_number, []).append(episode)

    return seasons


def _episode_key(episode):
    """
    Returns key used to sort episodes based on season and episode number.
    """
    return (episode['season_number'], episode['episode_number'])


def sorted_episodes(episodes):
    return sorted(episodes, key=_episode_key)


def series_finder(fetch_series, get_series, query_id):
    """
    Find series from the local database.
    """
    result_id = find_series(fetch_series, query_id)

    if result_id is None:
        raise Exception(u"no such series: {0}".format(query_id))

    series = get_series(result_id)

    if series is None:
        raise Exception(u"no such series (id): {0}".format(result_id))

    return series


def numeric_ranges(s):
    """
    Parse numeric ranges separated by comma.

    A numeric range can either be a single number, or a range separated by a
    dash.

    Overlaps are ignored since we are storing the result in a set.

    Example: 1,2,10-15,22 -> set([1,2,10,11,12,13,14,15,22])
    """
    numbers = set()

    parts = s.split(',')

    for p in parts:
        if '-' in p:
            left, right = p.split('-')
            numbers.update(range(int(left), int(right) + 1))
        else:
            numbers.add(int(p))

    return numbers


def numeric_range(s):
    if '-' not in s:
        raise Exception(
            ("not a valid range, expected '<begin>-<end>' but "
             "was '{0}'").format(s))

    begin, end = s.split("-", 1)

    try:
        return int(begin), int(end)
    except:
        raise Exception(
            "range does not contain numeric values '{0}'".format(s))


def find_next_episode(episodes, is_watched, ignored_seasons=set([0])):
    """
    Find the next episode that is not watched.

    :episodes: A list of all the episodes.
    :is_watched: A function returning true if the episode has been watched.
    :ignored_seasons: Seasons to ignore when checking if watched or not.

    Returns None if none can be found.
    """

    for episode in sorted_episodes(episodes):
        if episode['season_number'] in ignored_seasons:
            continue

        if is_watched(episode):
            continue

        if not episode['first_aired']:
            continue

        airdate = parse_datetime(episode['first_aired'])
        return episode, airdate

    return None


def find_last_episode(episodes, is_watched, ignored_seasons=set([0])):
    """
    Find the last episode that has been watched.

    :episodes: A list of all the episodes.
    :is_watched: A function returning true if the episode has been watched.
    :ignored_seasons: Seasons to ignore when checking if watched or not.

    Returns None if none can be found.
    """

    stored = None

    for episode in sorted_episodes(episodes):
        if episode['season_number'] in ignored_seasons:
            continue

        if is_watched(episode):
            stored = episode
            continue

        if not episode['first_aired']:
            continue

        break

    if stored is None:
        return None

    airdate = parse_datetime(stored['first_aired'])
    return stored, airdate


def has_aired_filter(now):
    """
    Utility function to create an episode filter for if an episode has been
    aired.
    """

    def __has_aired(e):
        if not e['first_aired']:
            return False
        return now >= parse_datetime(e['first_aired'])

    return __has_aired


# the following functions require a namespace since separating them is not
# worth the effort right now.


def local_series_finder(ns):
    """
    Find a series by id from the local database.

    :query_id The id of the series to find.
    """

    if ns.series_query is None:
        return None

    return series_finder(ns.series.find_series, ns.series.get, ns.series_query)


def local_episodes_finder(ns, series):
    """
    Find local episodes.

    Expectes series to be provided as an argument.
    """
    episodes = ns.series.get_episodes(series)

    if episodes is None:
        raise Exception("No episodes synced for series: {0}".format(
            series['series_name']))

    return list(find_episodes(ns, episodes))


def api_series_finder(ns):
    """
    Find a series by querying the remote database through the api.
    """
    if ns.language is None:
        raise Exception("language must be configured")

    def get_series(series_id):
        return ns.api.series(series_id, ns.language)

    return series_finder(ns.api.getseries, get_series, ns.series_query)


def with_resource(finder):
    def __with_resource(func):
        @functools.wraps(func)
        def __inner(ns, *args, **kw):
            try:
                resource = finder(ns, *args)
            except Exception as e:
                print ns.t.bold_red(str(e))
                return 1

            args = list(args) + [resource]
            return func(ns, *args, **kw)

        return __inner

    return __with_resource


def find_episodes(ns, episodes):
    """
    Find all episodes using the method deduced from the namespace.
    """
    special_find = None

    if hasattr(ns, 'last') and ns.last:
        special_find = find_last_episode

    if hasattr(ns, 'next') and ns.next:
        special_find = find_next_episode

    if special_find is not None:
        result = special_find(
            episodes, ns.series.is_episode_watched,
            ignored_seasons=ns.ignored_seasons)

        if result is None:
            print ns.t.bold_red(u"no episode is current")
            return

        episode, _ = result
        yield episode
        return

    for episode in episodes:
        if ns.seasons is not None:
            if episode['season_number'] not in ns.seasons:
                continue

        if ns.episodes is not None:
            if episode['episode_number'] not in ns.episodes:
                continue

        yield episode

    return



import unittest
import mock

from ontv.database import SetDB


class TestDatabaseSetDB(unittest.TestCase):
    def test_add(self):
        key = object()
        storage = mock.Mock()
        s = SetDB(storage, {})
        s.add(key)
        storage.setitem.assert_called_with(key, None)

    def test_remove(self):
        key = object()
        storage = mock.Mock()
        s = SetDB(storage, {key: None})
        s.remove(key)
        storage.delitem.assert_called_with(key)

    def test_pop(self):
        key = object()
        storage = mock.Mock()
        s = SetDB(storage, {key: None})
        self.assertEquals(key, s.pop())
        storage.delitem.assert_called_with(key)

    def test_compact(self):
        key = object()
        storage = mock.Mock()
        stats = storage.compact.return_value
        s = SetDB(storage, {key: None})
        self.assertEquals(stats, s.compact())
        self.assertTrue(storage.compact.called)
        self.assertItemsEqual([(key, None)], storage.compact.call_args[0][0])

    def test_db_size(self):
        storage = mock.Mock()
        size = storage.db_size.return_value
        s = SetDB(storage, {})
        self.assertEquals(size, s.db_size())



from ..utils import numeric_ranges
from ..utils import with_resource
from ..utils import local_series_finder
from ..utils import local_episodes_finder


@with_resource(local_series_finder)
@with_resource(local_episodes_finder)
def action(ns, series, episodes):
    changed = 0

    for episode in episodes:
        is_watched = ns.series.is_episode_watched(episode)

        name = u'{0[series_name]} S{1:02}E{2:02}: {3}'.format(
            series,
            episode['season_number'],
            episode['episode_number'],
            episode['episode_name']
        )

        if is_watched and not ns.unmark:
            print ns.t.bold_red(u"Already watched: {0}".format(name))
            continue

        if not is_watched and ns.unmark:
            print ns.t.bold_red(u"Not watched: {0}".format(name))
            continue

        ns.series.set_episode_watched(episode, (not ns.unmark))

        if not ns.unmark:
            print ns.t.bold_green(u"Marked: {0}".format(name))
        else:
            print ns.t.bold_green(u"Unmarked: {0}".format(name))

        changed += 1

    if changed == 0:
        print ns.t.bold_red(u"did not change any episodes")
        return 1

    return 0


def setup(parser):
    parser.add_argument(
        "series_query",
        metavar="<name|id>",
        help="The id or name of the series.",
    )

    parser.add_argument(
        "seasons",
        metavar="seasons",
        nargs='?',
        default=None,
        type=numeric_ranges,
        help="The season of the episode.",
    )

    parser.add_argument(
        "episodes",
        metavar="episodes",
        nargs='?',
        default=None,
        type=numeric_ranges,
        help="The episode number of the season.",
    )

    parser.add_argument(
        "--next",
        default=False,
        help="Mark the next episode not watched.",
        action='store_const',
        const=True,
    )

    parser.add_argument(
        "--last",
        default=False,
        help="Mark or unmark the last episode watched.",
        action='store_const',
        const=True,
    )

    parser.add_argument(
        '--ignored-seasons',
        '-i',
        metavar="<ranges>",
        help="Specify a list of seasons to ignore, defaults to '0'",
        default=set([0]),
        type=numeric_ranges,
    )

    parser.add_argument(
        "--unmark",
        default=False,
        help="The episode number of the season.",
        action='store_const',
        const=True,
    )

    parser.set_defaults(action=action)



from ..utils import local_series_finder
from ..utils import with_resource
from ..utils import numeric_ranges
from ..utils import group_episodes
from ..utils import local_episodes_finder
from ..format import print_series


@with_resource(local_series_finder)
@with_resource(local_episodes_finder)
def action(ns, series, episodes):
    print ns.t.bold_magenta(u"Details about series")
    print u""

    seasons = group_episodes(episodes)

    print_series(
        ns.t, series,
        seasons=seasons,
        ignored_seasons=ns.ignored_seasons,
        series_dao=ns.series)

    return 0


def setup(parser):
    parser.add_argument(
        "series_query",
        metavar="<name|id>",
        help="The id of the series to add",
    )

    parser.add_argument(
        "seasons",
        nargs="?",
        metavar="ranges",
        type=numeric_ranges,
        help="Filter out the specified season.",
        default=None,
    )

    parser.add_argument(
        "episodes",
        nargs="?",
        metavar="ranges",
        type=numeric_ranges,
        help="Filter out the specified episode.",
        default=None,
    )

    parser.add_argument(
        '--ignored-seasons',
        '-i',
        metavar="<ranges>",
        help="Specify a list of seasons to ignore, defaults to '0'",
        default=set([0]),
        type=numeric_ranges,
    )

    parser.add_argument(
        "--next",
        default=False,
        help="Show the next episode not watched.",
        action='store_const',
        const=True,
    )

    parser.add_argument(
        "--last",
        default=False,
        help="Show the last episode watched.",
        action='store_const',
        const=True,
    )

    parser.set_defaults(action=action)



from ..utils import write_yaml
from ..utils import with_resource
from ..utils import local_series_finder


@with_resource(local_series_finder)
def action(ns, series):
    print ns.t.bold_magenta(u"Synchronizing local database")
    print u""

    if not ns.api.is_authenticated():
        print ns.t.bold_red(
            "Cannot fully synchronize, api is not authenticated with a key!")
        print (
            ns.t.red("Set a value to the 'api_key' option in ") +
            ns.t.bold_red(ns.config_path)
        )
        print (
            ns.t.red("If you don't have an API key, register yourself at ") +
            ns.t.bold_red("http://thetvdb.com/?tab=register")
        )
        return 0

    if not ns.languages:
        with open(ns.languages_path, 'w') as fp:
            write_yaml(fp, ns.api.languages())

    if not ns.mirrors:
        print ns.t.bold_green("Downloading mirrors to {0}".format(
            ns.mirrors_path))

        with open(ns.mirrors_path, 'w') as fp:
            write_yaml(fp, ns.api.mirrors())

    if not series:
        series = ns.series.list_series()
    else:
        series = [series]

    for s in series:
        print ns.t.bold_cyan(
            u"Syncing: {0}".format(s['series_name'])
        )

        s, episodes = ns.api.series_all(
            s['id'], ns.language)

        ns.series.set_episodes(s, episodes)

    return 0


def setup(parser):
    parser.add_argument(
        "series_query",
        metavar="<name|id>",
        nargs='?',
        default=None,
        help="The id or name of the series to sync.",
    )

    parser.set_defaults(action=action)



import os
import argparse
import logging
import blessings
import random
import contextlib

from .api import TheTVDBApi
from .action.sync import setup as sync_setup, action as sync_action
from .action.search import setup as search_setup
from .action.add import setup as add_setup
from .action.remove import setup as remove_setup
from .action.list import setup as list_setup
from .action.show import setup as show_setup
from .action.mark import setup as mark_setup
from .action.next import setup as next_setup
from .action.compact import setup as compact_setup

from .dao import SeriesDAO

from .utils import write_yaml
from .utils import read_yaml

from .database import open_database

__version__ = "0.8.1"

log = logging.getLogger(__name__)

LOGGING_FORMAT = "%(asctime)s %(levelname)7s %(message)s"

TEMPLATE_CONFIGURATION = {
    "api_key": None,
    "language": "en",
}


class ColorScheme(object):
    def __init__(self, term, colors=dict()):
        self.__dict__['_term'] = term
        self.__dict__['_colors'] = dict(colors)

    def __setattr__(self, key, name):
        self._colors[key] = name

    def __getattr__(self, key):
        name = self._colors.get(key)

        if not name:
            raise KeyError("Color not defined for '{0}'".format(key))

        color = getattr(self._term, name, None)

        if color is None:
            raise KeyError("Missing color '{0}'".format(name))

        return color


def setup_parser(parser):
    parser.add_argument(
        '--libdir', '-d',
        metavar="<directory>",
        help="Library directory, defaults to $HOME/.ontv.",
        default=None,
    )

    parser.add_argument(
        '--loglevel', '-l',
        metavar="<level>",
        help="Logging level to use.",
        default=logging.WARNING,
    )

    parser.add_argument(
        '--debug',
        dest='loglevel',
        help="Use a debugging log level.",
        action='store_const',
        const=logging.DEBUG,
    )

    parser.add_argument(
        '--apikey',
        dest='api_key',
        metavar="<key>",
        help="API key to use",
        default=None,
    )

    parser.add_argument(
        '--language',
        dest='language',
        metavar="<language>",
        help="Language to use",
        default=None,
    )

    subparsers = parser.add_subparsers()

    sync_parser = subparsers.add_parser(
        "sync",
        help="Synchronize local database.",
    )
    sync_setup(sync_parser)

    search_parser = subparsers.add_parser(
        "search",
        help="Search for tv series.",
    )
    search_setup(search_parser)

    add_parser = subparsers.add_parser(
        "add",
        help="Add tv series.",
    )
    add_setup(add_parser)

    remove_parser = subparsers.add_parser(
        "remove",
        help="Remove tv series.",
    )
    remove_setup(remove_parser)

    list_parser = subparsers.add_parser(
        "list",
        help="List tv series you are watching.",
    )
    list_setup(list_parser)

    show_parser = subparsers.add_parser(
        "show",
        help="Show episodes in a series.",
    )
    show_setup(show_parser)

    mark_parser = subparsers.add_parser(
        "mark",
        help="Mark or unmark an episode as watched.",
    )
    mark_setup(mark_parser)

    next_parser = subparsers.add_parser(
        "next",
        help="Show the next episode to watch.",
    )
    next_setup(next_parser)

    compact_parser = subparsers.add_parser(
        "compact",
        help="Make the local database smaller.",
    )
    compact_setup(compact_parser)


def setup_ns(ns):
    home = os.environ.get("HOME")

    if ns.libdir is None:
        if not home:
            raise Exception("missing environment variable: HOME")

        ns.libdir = os.path.join(home, '.ontv')

    ns.mirrors_path = os.path.join(ns.libdir, 'mirrors.yaml')
    ns.languages_path = os.path.join(ns.libdir, 'languages.yaml')
    ns.config_path = os.path.join(ns.libdir, 'config.yaml')
    ns.db_path = os.path.join(ns.libdir, 'db')
    ns.series_db_path = os.path.join(ns.libdir, 'series')
    ns.episodes_db_path = os.path.join(ns.libdir, 'episodes')
    ns.watched_db_path = os.path.join(ns.libdir, 'watched')

    directories = [
        ns.libdir,
    ]

    for directory in directories:
        if not os.path.isdir(directory):
            log.info("Creating directory {0}".format(directory))
            os.mkdir(directory)

    ns.t = blessings.Terminal()

    if os.path.isfile(ns.config_path):
        log.debug("Loading configuration from {0}".format(ns.config_path))

        with open(ns.config_path) as fp:
            for key, value in read_yaml(fp).items():
                setattr(ns, key, value)
    else:
        log.info("Creating default configuration {0}".format(ns.config_path))

        with open(ns.config_path, 'w') as fp:
            write_yaml(fp, TEMPLATE_CONFIGURATION)

    if os.path.isfile(ns.mirrors_path):
        log.debug("Loading mirrors from {0}".format(ns.mirrors_path))

        with open(ns.mirrors_path) as fp:
            ns.mirrors = read_yaml(fp)
    else:
        ns.mirrors = []

    if os.path.isfile(ns.languages_path):
        log.debug("Loading mirrors from {0}".format(ns.languages_path))

        with open(ns.languages_path) as fp:
            ns.languages = read_yaml(fp)
    else:
        ns.languages = []

    if ns.languages:
        ns.abbrev_languages = [l['abbreviation'] for l in ns.languages]
    else:
        ns.abbrev_languages = []

    if ns.mirrors:
        ns.base_url = random.choice(ns.mirrors)['mirrorpath']
        log.debug("Picked mirror: {0}".format(ns.base_url))
    else:
        ns.base_url = None

    ns.api = TheTVDBApi(ns.api_key, base_url=ns.base_url)

    if ns.abbrev_languages and ns.language:
        if ns.language not in ns.abbrev_languages:
            raise Exception(
                "Language not valid, must be one of {0}".format(
                    ", ".join(ns.abbrev_languages)))
    else:
        ns.language = None

    ns.is_synced = bool(ns.abbrev_languages)

    ns.C = ColorScheme(ns.t)
    ns.C.range_before = 'green'
    ns.C.range_inside = 'yellow'
    ns.C.range_outside = 'red'
    ns.C.all_seen = 'magenta'
    ns.C.warning = 'red'
    ns.C.title = 'bold_magenta'
    ns.C.series_title = 'bold'


def main(args):
    parser = argparse.ArgumentParser(version="ontv " + __version__)

    setup_parser(parser)

    ns = parser.parse_args(args)

    logging.basicConfig(format=LOGGING_FORMAT, level=ns.loglevel)

    setup_ns(ns)

    databases = contextlib.nested(
        open_database(ns.db_path),
        open_database(ns.series_db_path),
        open_database(ns.episodes_db_path),
        open_database(ns.watched_db_path),
    )

    with databases as (db, series_db, episodes_db, watched_db):
        ns.databases = {
            "db": db,
            "series": series_db,
            "episodes": episodes_db,
            "watched": watched_db,
        }

        ns.series = SeriesDAO(db, series_db, episodes_db, watched_db)

        if not ns.is_synced and ns.action != sync_action:
            print ns.t.bold_red("Your first action should be 'sync'")
            return 1

        return ns.action(ns)
